//*********************************************************************************
//		CLASS:		Pluto95 - default package
//*********************************************************************************

/******************************************************************************
Calculation of Pluto coordinates, based on the work of J. Chapront and G. Francou, Bureau des Longitudes - France.
<BR/>This work can be found at <A HREF="xxx">xxx</A>. 
<BR/>In particular, two files from this directory were used : xxx and xxx.
<BR/>Licenced under the terms of the General Public Licence, available at 
<A HREF="xxx">xxx</A>
<BR/>Author : Thierry Graff, tigraff@chez.com
<BR/>History :
<LI>nov 06 2000 : creation</LI>
<LI>nov 06 2000 : made positions work</LI>
<LI>nov 19 2000 : debug error in velocities</LI>
@todo address of files, address of GPL
*********************************************************************************/

/*--------------- Extracts from file xxx ----------------------

-------------------
PLUTO TABLES (1995)
-------------------

J. Chapront, G. Francou
Bureau des Longitudes - France
Group : Dynamics of Solar System

December 1996

Authors' Address :

Bureau des Longitudes 
77, avenue Denfert Rochereau
F75014 Paris  France
Tel : (33) (1) 40.51.22.60
Fax : (33) (1) 46.33.28.34
E-mail : francou@bdl.fr

---------------
1. PRESENTATION
---------------

The tables of Pluto have been constructed by J. Chapront (BDL) with a new
method of approximation using frequency analysis as described in the paper :
Representation of planetary ephemerides by frequency analysis.
Application to the five outer planets.
Astron. & Astrophys.Suppl. Ser., 109, 191 (1995).

This representation uses the result of numerical integration DE200 of Jet
Propulsion Laboratory as a source :
Standish E. M., 1990,
The observational basis for JPL'DE200, the planetary ephemerides of the
Astronomical Almanac.
Astron. Astrophys., 233, 252.

The interval of validity is 146120 days.
Start : Jan 01 1700 0h  JD2341972.5
End :   Jan 24 2100 0h  JD2488092.5

The tables contain series which represent the heliocentric rectangular
coordinates of Pluto as functions of time.

The reference frame is defined with dynamical equinox and equator J2000
(DE200).

The time scale is Barycentric Dynamical Time (TDB).

----------------------------
4. PRECISION AND TEST-VALUES
----------------------------

Hereunder, we give 5 test-values computed with the subroutine pluto.sub.
In this output the number of decimals is intentionally superabundant for
checking the use of this subroutine.

The numerical accuracy can be evaluated with the largest discrepancy obtained
in a comparison with the ephemerides DE200 on the period 1700-2100 :
for the positions  : 5 E-7 au (0.003"),
for the velocities : 6 E-9 au/d.

PLUTO  Heliocentric Rectangular Coordinates (mean equinox and equator J2000)

Date:  Jan  01   1700    H : 00h 00m 00s (TDB)    JD:   2341972.5000000
X :-25.48366603086599    Y : 22.25190224179014    Z : 14.61666566142614   au
X': -0.00140296544832    Y': -0.00253543942176    Z': -0.00036577359317   au/d

Date:  Jan  02   1800    H : 06h 00m 00s (TDB)    JD:   2378497.7500000
X : 36.33316699469712    Y :-11.84871881208418    Z :-14.64079073464049   au
X':  0.00151098228705    Y':  0.00214812030172    Z':  0.00021249511616   au/d

Date:  Jan  03   1900    H : 12h 00m 00s (TDB)    JD:   2415023.0000000
X : 10.29158303131287    Y : 44.52906466047693    Z : 10.79081191605171   au
X': -0.00216104614307    Y': -0.00004877516272    Z':  0.00063748726618   au/d

Date:  Jan  04   2000    H : 18h 00m 00s (TDB)    JD:   2451548.2500000
X : -9.86615874601937    Y :-27.98285304568784    Z : -5.75779357947923   au
X':  0.00302900782509    Y': -0.00112671144850    Z': -0.00126494662037   au/d

Date:  Jan  05   2100    H : 00h 00m 00s (TDB)    JD:   2488073.5000000
X : 39.67448463874504    Y : 28.47968765660414    Z : -3.06796133066342   au
X': -0.00097971861494    Y':  0.00171018575529    Z':  0.00082844820875   au/d
*/

public class Pluto95{

	//***************** Constants *************************
  /** Validity start date (julian days) */
	private static final double VALIDITY_START	= 2341972.5;
  /** Validity end date (julian days) */
	private static final double VALIDITY_END	= 2488092.5;
  /** Index of last periodic term - WARNING : index begins to 1. */
	private static final int N_PERIODIC	= 82;
  /** Index of last Poisson term order 1 */
	private static final int N_POISSON1	= 101;
  /** Index of last Poisson term order 2 */
	private static final int N_POISSON2	= 106;

  /*************************************************************
	Main function for autonom use.
	*************************************************************/
  public static void main(String[] args){
  
    double[] res = new double[6];
    double jd = new Double(args[0]).doubleValue();
    int ret = calcPluto(jd, res, true);
    
    String names[] = {"X", "Y", "Z", "VX", "VY", "VZ"};
    for (int i=0;i<6;i++)
      System.out.println(names[i] + " = " + res[i]);
  }// end main

  /****************************************************************
	Calculation of Pluto coordinates and velocities.
	@param jd Julian day.
	@param res Array where result of calculations are stored.
  @param Velocities indicates wether velocities should be calculated.
	@return code : 0 : OK;
                 1 : out of date validity;
                 2 : res argument not correct.
	*************************************************************/
  public static int calcPluto(double jd, double[] res, boolean velocities){
  
    if (jd < VALIDITY_START || jd > VALIDITY_END) return 1;
    if (res.length != 6) return 2;
    res[0]=res[1]=res[2]=res[3]=res[4]=res[5]=0;

    double dt = VALIDITY_END - VALIDITY_START;
    double x = 2*(jd - VALIDITY_START)/dt - 1;
    double x2 = x*x;
    double x3 = x2*x;
    double fx = x*dt/2;
    
    // *** 1 - Positions ***
    
    // Secular terms
    res[0] = AX[0] + AX[1]*x + AX[2]*x2 + AX[3]*x3; 
    res[1] = AY[0] + AY[1]*x + AY[2]*x2 + AY[3]*x3; 
    res[2] = AZ[0] + AZ[1]*x + AZ[2]*x2 + AZ[3]*x3; 
    
    // For X, Y and Z,  terms CX and SX :
    // CX[i] = CXSX[i][0] ; SX[i] = CXSX[i][1] for periodic and Poisson order 2 terms
    // CX[i] = CXSX[i][1] ; SX[i] = CXSX[i][0] for Poisson order 1 terms
    // (inversion in original file left here)

    double cf, sf;

    // Periodic terms
    for (int i = 0; i < N_PERIODIC; i++){
      cf = Math.cos(FQ[i]*fx);
      sf = Math.sin(FQ[i]*fx);
      res[0] += CXSX[i][0]*cf + CXSX[i][1]*sf;
      res[1] += CYSY[i][0]*cf + CYSY[i][1]*sf;
      res[2] += CZSZ[i][0]*cf + CZSZ[i][1]*sf;
    }

    // Poisson terms, order 1
    for (int i = N_PERIODIC; i < N_POISSON1; i++){
      cf = Math.cos(FQ[i]*fx);
      sf = Math.sin(FQ[i]*fx);
      res[0] += (CXSX[i][1]*cf + CXSX[i][0]*sf)*x;
      res[1] += (CYSY[i][1]*cf + CYSY[i][0]*sf)*x;
      res[2] += (CZSZ[i][1]*cf + CZSZ[i][0]*sf)*x;
    }

    // Poisson terms, order 2
    for (int i = N_POISSON1; i < N_POISSON2; i++){
      cf = Math.cos(FQ[i]*fx);
      sf = Math.sin(FQ[i]*fx);
      res[0] += (CXSX[i][0]*cf + CXSX[i][1]*sf)*x2;
      res[1] += (CYSY[i][0]*cf + CYSY[i][1]*sf)*x2;
      res[2] += (CZSZ[i][0]*cf + CZSZ[i][1]*sf)*x2;
    }
    
    // *** 2 - Velocities ***
    if (velocities){
      double wt = 2/dt;

      // Secular terms
      res[3] = (AX[1] + 2*AX[2]*x + 3*AX[3]*x2)*wt; 
      res[4] = (AY[1] + 2*AY[2]*x + 3*AY[3]*x2)*wt; 
      res[5] = (AZ[1] + 2*AZ[2]*x + 3*AZ[3]*x2)*wt; 
    
    // Periodic terms
      for (int i = 0; i < N_PERIODIC; i++){
        cf = Math.cos(FQ[i]*fx);
        sf = Math.sin(FQ[i]*fx);
        res[3] += FQ[i]*(CXSX[i][1]*cf - CXSX[i][0]*sf);
        res[4] += FQ[i]*(CYSY[i][1]*cf - CYSY[i][0]*sf);
        res[5] += FQ[i]*(CZSZ[i][1]*cf - CZSZ[i][0]*sf);
      }

    // Poisson terms, order 1  (remember inversion of CX and SX for order 1)
      for (int i = N_PERIODIC; i < N_POISSON1; i++){
        cf = Math.cos(FQ[i]*fx);
        sf = Math.sin(FQ[i]*fx);
        res[3] += FQ[i]*(CXSX[i][0]*cf - CXSX[i][1]*sf)*x + wt*(CXSX[i][1]*cf + CXSX[i][0]*sf);
        res[4] += FQ[i]*(CYSY[i][0]*cf - CYSY[i][1]*sf)*x + wt*(CYSY[i][1]*cf + CYSY[i][0]*sf);
        res[5] += FQ[i]*(CZSZ[i][0]*cf - CZSZ[i][1]*sf)*x + wt*(CZSZ[i][1]*cf + CZSZ[i][0]*sf);
      }

    // Poisson terms, order 2
    for (int i = N_POISSON1; i < N_POISSON2; i++){
      cf = Math.cos(FQ[i]*fx);
      sf = Math.sin(FQ[i]*fx);
      res[3] += FQ[i]*(CXSX[i][1]*cf - CXSX[i][0]*sf)*x2 + 2*wt*(CXSX[i][0]*cf + CXSX[i][1]*sf)*x;
      res[4] += FQ[i]*(CYSY[i][1]*cf - CYSY[i][0]*sf)*x2 + 2*wt*(CYSY[i][0]*cf + CYSY[i][1]*sf)*x;
      res[5] += FQ[i]*(CZSZ[i][1]*cf - CZSZ[i][0]*sf)*x2 + 2*wt*(CZSZ[i][0]*cf + CZSZ[i][1]*sf)*x;
      }

    }// end velocities
    
    // Convert to a.u.
    for (int i=0; i<6 ; i++)
      res[i] /= 1.0e10;

    return 0;

  }// end calcPluto

  //******************************** DATA *****************************************
  
  /** TABLE FQ : Frequencies. Coordinates X, Y, Z (radian/day) */
  private static double[] FQ = {
    // Periodic terms
    0.0000645003954767,
    0.0001083248054773,
    0.0001302772403167,
    0.0001647868659960,
    0.0001935009111902,
    0.0002223740247147,
    0.0003032575201026,
    0.0003259246239385,
    0.0003564763034914,
    0.0004265811293132,
    0.0004503959517513,
    0.0004638675148284,
    0.0005009272733421,
    0.0005163593863414,
    0.0005578826828210,
    0.0005882795362847,
    0.0006450023602974,
    0.0007097635821639,
    0.0007630643253588,
    0.0007740033551209,
    0.0008385031396726,
    0.0008950591609720,
    0.0009545118163938,
    0.0010255417569600,
    0.0010826728325744,
    0.0011680358909203,
    0.0012405125052369,
    0.0012931805883876,
    0.0013460706181008,
    0.0014190059530383,
    0.0014394705053002,
    0.0014502634075377,
    0.0014992014575181,
    0.0015434430430867,
    0.0016000710611098,
    0.0016562809940875,
    0.0017275924266291,
    0.0017454042542465,
    0.0018215079641428,
    0.0018694826929211,
    0.0019274630193251,
    0.0020276790928706,
    0.0021822818660433,
    0.0022885289854970,
    0.0023167646379420,
    0.0023445464874575,
    0.0024069306189938,
    0.0024473146628449,
    0.0024778027974419,
    0.0025244208011161,
    0.0025682157855485,
    0.0026028617439482,
    0.0026544444009919,
    0.0026987455959123,
    0.0027308225916697,
    0.0027735113723168,
    0.0028728385464030,
    0.0029001725379479,
    0.0029379670182566,
    0.0029750359447782,
    0.0031326820696785,
    0.0031822107498712,
    0.0031931048857857,
    0.0032268922327691,
    0.0034657232066225,
    0.0037838581645670,
    0.0038055149432355,
    0.0038631344783149,
    0.0039129259467328,
    0.0040311445462510,
    0.0040607542008930,
    0.0041490103414206,
    0.0043500678052272,
    0.0046321937641054,
    0.0058000376725240,
    0.0091460971544658,
    0.0091560629947357,
    0.0172021239411871,
    0.0182919855069063,
    0.0279624510118796,
    0.0344040996177640,
    0.0714245719830324,

    // Poisson terms, order 1
    0.0001083248054773,
    0.0001647868659960,
    0.0003032575201026,
    0.0003259246239385,
    0.0003564763034914,
    0.0005009272733421,
    0.0005882795362847,
    0.0007097635821639,
    0.0007630643253588,
    0.0008950591609720,
    0.0011680358909203,
    0.0014502634075377,
    0.0017454042542465,
    0.0029001725379479,
    0.0031822107498712,
    0.0040607542008930,
    0.0043500678052272,
    0.0091460971544658,
    0.0279624510118796,
    
    // Poisson terms, order 2
    0.0001083248054773,
    0.0003032575201026,
    0.0011680358909203,
    0.0043500678052272,
    0.0279624510118796
  }; // end FQ

  /** TABLE AX : Secular Terms. Coordinate X (1.E10 au) */
  private static double[] AX ={
    98083308510.,
    -1465718392.,
    11528487809.,
    55397965917.
  }; // end AX

  /** TABLE AY : Secular Terms. Coordinate Y (1.E10 au) */
  private static double[] AY ={
    101846243715.,
    57789.,
    -5487929294.,
    8520205290.
  }; // end AY

  /** TABLE AZ : Secular Terms. Coordinate Z (1.E10 au) */
  private static double[] AZ ={
    2183700004.,
    433209785.,
    -4911803413.,
    -14029741184.
  }; // end AZ

  /** TABLES CX and SX : Periodic and Poisson Terms. Coordinate X (1.E10 ua) */
  private static double[][] CXSX ={
    // Periodic Terms
    { -16338582222., -308294137468. },
    { -5995086437., -68820910480. },
    { 23663880362., 28346466257. },
    { 10304632056., -1755658975. },
    { -3996936944., 7818660837. },
    { -4136465568., -1098895702. },
    { 1188702881., -1192462299. },
    { -621434363., -772129982. },
    { 566898160., 1061702581. },
    { -75880391., -639572722. },
    { 576146406., 1128327488. },
    { -659684298., -423570428. },
    { 451962774., -175317704. },
    { -153724334., 251601606. },
    { -603163280., -869448807. },
    { 364764379., 551228298. },
    { 193062130., 87807522. },
    { 161493959., -11540541. },
    { 1167349082., -103236703. },
    { -1417467887., 92638954. },
    { 15325240., -3624991. },
    { -3624391., 1004975. },
    { -587306., 304396. },
    { 132022., -56532. },
    { -106501., 55554. },
    { 228373., -799096. },
    { -95106., 56947. },
    { 56299., -48016. },
    { -48339., 50599. },
    { 803937., -680660. },
    { -6172744., 5858452. },
    { -18962749., 38125648. },
    { 133022., -109460. },
    { -25964., 18684. },
    { 7111., -5269. },
    { -4998., 2771. },
    { 32034., -6814. },
    { -29666., 47130. },
    { -1983., 1192. },
    { 114., -1387. },
    { 191., 379. },
    { -1063., -612. },
    { 419., -52. },
    { 346., 813. },
    { 5059., -4354. },
    { -81., -2275. },
    { 1408., 685. },
    { 2964., -1352. },
    { -5364., 4681. },
    { 1509., -1908. },
    { -4924., -6530. },
    { 2954., 8667. },
    { 2034., 1675. },
    { -5199., 874. },
    { 604., 898. },
    { -1247., 965. },
    { 4576., -7124. },
    { -350741., -1145389. },
    { -4023., 2931. },
    { 1147., -618. },
    { -38., -34. },
    { -99., -6562. },
    { -11686., 8038. },
    { 1129., -697. },
    { 582., -8. },
    { -83., 12. },
    { -97., -267. },
    { 431., -131. },
    { -134., 304. },
    { -323., -756. },
    { -292., -103. },
    { 195., -250. },
    { 39068., 19816. },
    { 523., -596. },
    { -1747., 576. },
    { 3135., 4122. },
    { -619., 65. },
    { -12095., -27900. },
    { 6., 217. },
    { 18476., -137. },
    { -130., -269. },
    { -438., 531. },

    // Poisson Terms Order 1
    { -24338350765., 102345278799. },
    { 11210995713., -9329130892. },
    { 2793567155., 1484339404. },
    { -776019789., 472660593. },
    { 1528323591., -581239444. },
    { -249354416., 1016663241. },
    { 1127608109., -1054199614. },
    { -667692329., 99039105. },
    { -1570766679., -52190030. },
    { -9724425., -3394173. },
    { 26552., -16529. },
    { 3332520., 3102430. },
    { -27607., 2286. },
    { -11696., -10955. },
    { -7297., -5293. },
    { -104., -654. },
    { -184., 124. },
    { -455., -85. },
    { -16., 29. },

    // Poisson Terms Order 2
    { 418209651., 39813894679. },
    { -1191875710., 3633087275. },
    { -823081., 522728. },
    { -558., -320. },
    { -1091., -1401. }
  }; // end CXSX

  /** TABLES CY and SY : Periodic and Poisson Terms. Coordinate Y (1.E10 ua) */
  private static double[][] CYSY ={
    // Periodic Terms
    { 299584895562., -53545027809. },
    { 75951634908., -8838029861. },
    { -36135662843., 23553788174. },
    { 18125610071., 13775798112. },
    { -20398008415., -6068121593. },
    { 6125780503., -2853107588. },
    { -162559485., 750355551. },
    { 4352425804., -82067770. },
    { -3819676998., 230091832. },
    { 1168107376., -259838942. },
    { -5041323701., 197944074. },
    { 4093828501., 27141006. },
    { -1727274544., -105334544. },
    { 134214260., 95175918. },
    { 5033950069., -139461973. },
    { -3071449401., 80593104. },
    { -1190419055., -5126842. },
    { -775881742., -21953793. },
    { -5524713888., -163767784. },
    { 6803228005., 192436228. },
    { -65675611., -2479113. },
    { 15155413., 561687. },
    { 2009509., 121909. },
    { -389682., -30275. },
    { 275571., 16333. },
    { 474366., 68105. },
    { 132163., 24081. },
    { -81550., -11228. },
    { 69996., 667. },
    { -706470., -73047. },
    { 4777898., 1007089. },
    { -44002785., -22814549. },
    { -58735., 434. },
    { 7624., 1013. },
    { -1922., 710. },
    { -729., 1100. },
    { -1733., -4598. },
    { -35642., 1990. },
    { -586., 564. },
    { -258., 828. },
    { -368., -1119. },
    { 1286., -1249. },
    { -136., -597. },
    { 883., 227. },
    { 2673., 5467. },
    { 331., 801. },
    { 50., -2029. },
    { 178., -1892. },
    { 2901., 4713. },
    { -654., -459. },
    { -8972., 1757. },
    { 3034., -9303. },
    { 1113., -2357. },
    { 570., 7679. },
    { -72., -2953. },
    { 1950., 629. },
    { 8550., 5011. },
    { 1047593., -333905. },
    { -2348., -2388. },
    { 313., 415. },
    { 432., 139. },
    { 6765., -5726. },
    { -8240., -4583. },
    { 335., 310. },
    { 140., 681. },
    { -833., -107. },
    { 252., 301. },
    { -210., -525. },
    { 366., 198. },
    { -920., -379. },
    { 1215., -230. },
    { -217., -64. },
    { -17780., 36069. },
    { 581., 459. },
    { -560., -1596. },
    { -4131., 2509. },
    { 390., -146. },
    { 25613., -11081. },
    { -206., 4. },
    { 1850., 15764. },
    { 171., -147. },
    { -471., -362. },

    // Poisson Terms Order 1
    { 117449924600., 26437625772. },
    { -7691661502., -12674907683. },
    { -4771148239., -1067899665. },
    { 3733883366., -2082744. },
    { -7081845126., -43195632. },
    { 3502526523., 211912497. },
    { -8115570206., -108307161. },
    { 3607883959., -63033809. },
    { 7690328772., -203850703. },
    { 37384011., -1672332. },
    { -164319., 7136. },
    { -2859257., 803655. },
    { 1593., -10985. },
    { -11997., 9126. },
    { -6476., 3317. },
    { 1419., -151. },
    { 34., 160. },
    { 232., 138. },
    { 32., -27. },

    // Poisson Terms Order 2
    { -36463065062., 2752753498. },
    { -5816560445., -672124207. },
    { 1576292., 154239. },
    { -21., -400. },
    { -295., 372. }
  }; // end CYSY

  /** TABLES CZ and SZ : Periodic and Poisson Terms. Coordinate Z (1.E10 ua) */
  private static double[][] CZSZ ={
    // Periodic Terms
    { 98425296138., 76159403805. },
    { 25475793908., 17987340882. },
    { -18424386574., -1193982379. },
    { 2645968636., 4828308190. },
    { -5282207967., -4248985438. },
    { 3278235471., -559147671. },
    { -425422632., 593594960. },
    { 1526641086., 208799497. },
    { -1323182752., -249913200. },
    { 235873266., 115051024. },
    { -1617466723., -282588988. },
    { 1557465867., 135883560. },
    { -848586296., 23091693. },
    { 218182986., -49187976. },
    { 1636044515., 223956575. },
    { -1001334243., -137344299. },
    { -455739370., -28188872. },
    { -348173978., -2636274. },
    { -2511254281., -14202661. },
    { 3062521470., 25488216. },
    { -32079379., 419837. },
    { 7597939., -150966. },
    { 1138566., -64906. },
    { -238849., 3719. },
    { 192377., -2226. },
    { 83169., 86321. },
    { 148694., -15970. },
    { -92489., 16609. },
    { 87116., -15782. },
    { -1281070., 200300. },
    { 9950106., -1500491. },
    { -25105642., -9161491. },
    { -171749., 37481. },
    { 31035., -4616. },
    { -8648., 224. },
    { 5360., -1027. },
    { -30345., 5220. },
    { 11482., -6976. },
    { 1322., -267. },
    { -467., 556. },
    { 96., -23. },
    { 894., -711. },
    { -381., -122. },
    { -583., -97. },
    { 2525., 2440. },
    { -569., 786. },
    { 226., -806. },
    { -2039., -167. },
    { 3728., -156. },
    { -1540., 572. },
    { 42., 2532. },
    { -3144., -4582. },
    { 658., -1178. },
    { 220., 875. },
    { 1848., -558. },
    { 678., 781. },
    { -7289., 3230. },
    { 463291., -116132. },
    { 3945., -1440. },
    { -1141., 438. },
    { -26., 176. },
    { -10607., 1072. },
    { 11458., -5850. },
    { -1005., 418. },
    { 120., 267. },
    { -301., 60. },
    { 135., 134. },
    { -186., -85. },
    { 118., -59. },
    { 30., 112. },
    { 197., -168. },
    { -182., -89. },
    { -8585., 14986. },
    { 240., 190. },
    { -226., -685. },
    { -2049., 1018. },
    { 283., -48. },
    { 11109., -4807. },
    { -100., 0. },
    { -842., 7066. },
    { 71., -54. },
    { -181., -229. },

    // Poisson Terms Order 1
    { 44126663549., -22591501373. },
    { -5626220823., -1138977908. },
    { -2536450838., -782718600. },
    { 1536292657., -141483824. },
    { -2916144530., 159033355. },
    { 949074586., -246222739. },
    { -2842935040., 287284767. },
    { 1500396857., -48002332. },
    { 3415136438., -41114335. },
    { 19702076., 578004. },
    { -46995., -8420. },
    { -5801645., -766779. },
    { 33470., 957. },
    { 17674., 5780. },
    { 7355., 4141. },
    { 199., 417. },
    { 11., -8. },
    { 205., 65. },
    { 33., -22. },

    // Poisson Terms Order 2
    { -11656050047., -11127973411. },
    { -1186276469., -1310869292. },
    { 1388681., -164753. },
    { 201., -107. },
    { 561., 284. }
  }; // end CZSZ

} //end class Pluto95
